package de.hs_rm.cs.swt.knauf.unittestmittimer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import de.hs_rm.cs.swt.knauf.unittestmittimer.timer.ITimer;

/**Logik-Klasse der unglaublich tollen Anwendung fr den Timer-Unit-Test:
 * sie enthlt einen @see {@link ITimer} , der nach dem Aufruf von "startTimer" 
 * regelmig auslst.
 * 
 * Intern zhlt die Klasse nur die Anzahl an Timer-Ticks hoch.
 * 
 * Verwendung: "startTimer" aufrufen. Danach luft der Timer (entweder ein automatisch
 * nach x Millisekunden auslsender Timer oder in der Unit-Test-Verwendung ein
 * manuell auslsender Timer - @see {@link de.hs_rm.cs.swt.knauf.unittestmittimer.timer.ManuellerTimer} )
 * 
 * @author Wolfgang Knauf
 *
 */
public class UnitTestMitTimerLogik
{
  /**ber "startTimer" gesetzter ActionListener, der beim tick des Timers aufgerufen wird.
   * 
   */
  private ActionListener timerListener;

  /**ber "startTimer" gesetzter Timer: wenn er tickt,
   * wird der ActionListener "timerListener" auslst. 
   * 
   */
  private ITimer timer;
  
  /**Anzahl der gesamten Timer-Ticks. Wird bei jedem Timer-Tick hochgezhlt.
   * Vor dem Aufruf von "startTimer" ist sie "0".
   * 
   */
  private int intTimerTicks = 0;

  /**Initialisieren dieser Klasse mit dem bergebenen Timer und Starten desselben.
   * 
   * Im Timer wird auerdem der bergebene ActionListener registriert: bei jedem
   * Timer-Tick wird die "actionPerformed"-Methode des Timers ausgelst. 
   * 
   * @param timer Zu verwendender Timer (entweder ein Timer, der alle x Millisekunden
   * auslst, oder ein manueller Timer in der Unit-Test-Verwendung). Not null.
   * Auf dem Timer sollte KEIN ActionListener registriert sein.
   * @param timerListener Dieser ActionListener wird aufgerufen, wenn
   * der Timer tickt. Not null
   */
  public void startTimer(ITimer timer, ActionListener timerListener)
  {
    if (timer == null)
    {
      throw new IllegalArgumentException("Timer ist null");
    }
    if (timerListener == null)
    {
      throw new IllegalArgumentException("ActionListener fr Timer ist null");
    }
    
    this.timerListener = timerListener;
    this.timer = timer;
    this.timer.addTimerListener(new ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {

        tick();
      }
    });
    
    this.timer.start();
  }

  /**Internes Timer-Tick (kommt erst nach "startTimer"): Auslsen des "timerListener"
   * 
   */
  private void tick()
  {
    System.out.println("ticked");

    ++this.intTimerTicks;
    this.timerListener.actionPerformed(null);

  }
  
  /**Abrufen der Anzahl von Timer-Ticks seit dem Starten des Timers.
   * Wenn der Timer noch nicht gestartet wurde, dann kommt "0"
   * 
   * @return Anzahl von Timer-Ticks
   */
  public int getTimerTickCount()
  {
    return this.intTimerTicks;
  }
}
