package de.hs_rm.cs.swt.knauf.unittestmittimer;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;

import de.hs_rm.cs.swt.knauf.unittestmittimer.timer.SwingTimer;

/**Hauptfenster der Timer-Test-Anwendung: 
 * ber Klicken eines Buttons wird eine Instanz von @see {@link UnitTestMitTimerLogik}
 * erzeugt und innerhalb derselben ein Swing-Timer gestaret.
 * Ab diesem Zeitpunkt gibt ein Label die Anzahl an Timer-Ticks innerhalb der Logik aus.
 * 
 * @author Wolfgang Knauf
 *
 */
public class UnitTestMitTimerMainFrame extends JFrame
{

  /**Eclipse sagt: soll so sein...
   * 
   */
  private static final long serialVersionUID = 1L;
  
  /**Label fr die Ausgabe der Anzahl an Timer-Ticks.
   * Wird erst nach Button-Klick eingeblendet
   * 
   */
  private JLabel labelTimerStatus;
  /**Button startet die Spiellogik bzw. den Timer darin.
   * Beim Klicken wird er aus- und das Label eingeblendet.
   */
  private JButton buttonTimerStart;

  /**Die Logik der Anwendung, hier mit Swing-Timer.
   * 
   */
  private UnitTestMitTimerLogik logik;

  /**Kapsel um den Swing-Timer, der alle x Millisekunden auslst
   * und ein Refresh des Labeltexts bewirkt.
   * 
   */
  private SwingTimer realTimer;
  
  /**Konstruktor erzeugt das Fenster. 
   * Der User muss auf den Start-Button klicken, damit die Logik losluft
   * 
   */
  public UnitTestMitTimerMainFrame()
  {
    this.setTitle("Application mit Timer");
    this.labelTimerStatus = new JLabel();
    labelTimerStatus.setText("Timer nicht gestartet");
    labelTimerStatus.setSize(200, 20);
    labelTimerStatus.setPreferredSize(new Dimension(200, 20));
    this.add(labelTimerStatus);

    this.buttonTimerStart = new JButton("Start");
    buttonTimerStart.setSize(50, 20);
    buttonTimerStart.setPreferredSize(new Dimension(50, 20));
    buttonTimerStart.setLocation(100, 0);
    buttonTimerStart.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent arg0)
      {
        buttonTimerStart.setVisible(false);
        labelTimerStatus.setVisible(true);

        logik.startTimer(realTimer, new ActionListener()
        {
          @Override
          public void actionPerformed(ActionEvent arg0)
          {
            timerTicked();
          }
        });

      }
    });
    this.add(buttonTimerStart);
    // Hide button
    this.labelTimerStatus.setVisible(false);

    this.setMinimumSize(new Dimension(200, 100));
    this.setPreferredSize(new Dimension(200, 100));
    pack();
    setVisible(true);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    //Echten Timer erzeugen:
    this.realTimer = new SwingTimer(3000);
    
    this.logik = new UnitTestMitTimerLogik();
  }

  /**Die Logik sagt "der Timer hat getickt": Ausgabe der Tick-Anzahl im Label.
   * 
   */
  public void timerTicked()
  {
    this.labelTimerStatus.setText("Timer ticked " + this.logik.getTimerTickCount() + " times");

  }
}
