package de.hs_rm.cs.swt.knauf.unittestmittimer.test;

import static org.junit.Assert.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.junit.Before;
import org.junit.Test;

import de.hs_rm.cs.swt.knauf.unittestmittimer.UnitTestMitTimerLogik;
import de.hs_rm.cs.swt.knauf.unittestmittimer.timer.ManuellerTimer;

/**Unit-Test fr die @see {@link UnitTestMitTimerLogik}:
 * hier wird ein @see {@link ManuellerTimer} erzeugt, dessen Tick-Event
 * hndisch durch Aufrufen seiner Methode "tick" ausgelst wird.
 * Dadurch umgeht man Timing-Probleme, die sich bei Verwendung eines Swing-Timers
 * automatisch ergeben.
 * 
 * Der Test besteht darin, zu prfen, ob nach 3 Ticks die Logik wirklich
 * den Zustand "Timer hat 3-mal ausgelst" besteht
 * 
 *  Im "setUp" wird die Spiellogik initialisiert, so dass die Test-Methode
 *  direkt mit Timerticks anfangen kann.
 * 
 * @author Wolfgang Knauf
 *
 */
public class UnitTestMitTimerTest
{
  private UnitTestMitTimerLogik logik;

  private ManuellerTimer timerManuell;

  /**Vor Ausfhrung jeder Test-Methode: Spiellogik initialisieren.
   * Hier erfolgt die Initialisierung mittels eines manuellen Timers.
   * 
   * @throws Exception Jeglicher Fehler - hier ist nix zu erwarten.
   */
  @Before
  public void setUp() throws Exception
  {
    this.logik = new UnitTestMitTimerLogik();

    // Manuellen Timer reintun:
    this.timerManuell = new ManuellerTimer();
    this.logik.startTimer(this.timerManuell,  new ActionListener()
    {
      @Override
      public void actionPerformed(ActionEvent e)
      {
        timerTicked();
      }
    });
  }

  /**Ausfhren des Tests "Timer dreimal ticken lassen und danach prfen,
   * ob die Logik den richtigen Tick-Count hat".
   * 
   */
  @Test
  public void test()
  {
    //Timer dreimal ticken lassen!
    this.timerManuell.tick();
    this.timerManuell.tick();
    this.timerManuell.tick();
    
    
    //Check: der interne Status der Logik-Klasse sollte jetzt besagen "Timer hat dreimal ausgelst"
    assertEquals(3,  this.logik.getTimerTickCount());
  }

  /**Timer in Logik tickt nach Aufruf der Methode "tick" des manuellen Timers:
   * hier passiert nix, es muss nur eine Methodenimplementierung da sein.
   * 
   * Man knnte aber auch im Unit-Test mitzhlen, wie oft diese Methode aufgerufen wurde,
   * und auch diesen Tick-Zhler in der "test"-Methode abprfen!
   * 
   */
  private void timerTicked()
  {
    // Nscht tun...
    System.out.println ("Manueller Timer hat getickt...");
    
  }

}
