package de.hs_rm.cs.swt.knauf.unittestmittimer.timer;

import java.awt.event.ActionListener;

/**Diese Implementierung von @see {@link ITimer} kapselt einen 
 * "manuellen Timer", der in Unit-Tests verwendet werden kann: er lst
 * nicht automatisch aus, sondern nur nach Aufruf der in dieser Klasse spezifischen 
 * Methode "tick":
 * jedes Aufrufen von "tick" ruft die Methode "actionPerformed"
 * auf dem ber "addTimerListener" bergebenen ActionListener aus.
 * 
 * Dadurch ist es in UnitTests mglich, den Fall "ein Timer hat x mal getickt" 
 * exakt nachzustellen, ohne sich auf die Ungenauigkeiten von Zeitsynchronisierung
 * in UnitTest verlassen zu mssen.
 * 
 * @author Wolfgang Knauf
 *
 */
public class ManuellerTimer implements ITimer
{
  /**Von auen gesetzter ActionListener, der beim Aufruf der Methode "tick"
   * das "Timer hat ausgelst"-Event bekommt.
   * 
   */
  private ActionListener timerTickListener;

  /**{@inheritDoc}
   * 
   */
  @Override
  public void addTimerListener(ActionListener actionListener)
  {
    this.timerTickListener = actionListener;
    
  }
  
  /**Starten des Timers: macht hier nichts, da es keinen automatisch
   * ablaufenden Timer gibt.
   * 
   */
  @Override
  public void start()
  {
    //Nix zu tun!
    
  }
  
  /**Spezielle Methode fr diesen Timer: da der Timer nichts automatisch macht,
   * lst nur diese Methode die "actionPerformed"-Methode des TimerListener aus.
   * Jeder Aufruf dieser Methode entspricht somit einem Auslsen eines automatischen Timers.
   * 
   */
  public void tick()
  {
    if (this.timerTickListener != null)
    {
      this.timerTickListener.actionPerformed(null);
    }
  }
}
