package de.hs_rm.cs.swt.knauf.unittestmittimer.timer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

/**Diese Implementierung von @see {@link ITimer} kapselt einen 
 * @see javax.swing.Timer der alle x Millisekunden auslst.
 * 
 * Im Konstruktor wird das Timerinterval definiert.
 * Nach Aufruf der Methode "start" luft der Timer und ruft alle x Millisekunden
 * die Methode "actionPerformed" eines ActionListener auf, der 
 * ber die Methode "addTimerListener" bergeben wurde.
 * 
 * @author Wolfgang Knauf
 *
 */
public class SwingTimer implements ITimer
{
  /**Dies ist der eigentliche Timer. Wird im Konstruktor initialisert
   * und durch die Methode "start" gestartet
   */
  private Timer timer;
  
  /**Von auen gesetzter ActionListener, der beim Ticken des Timers
   * das "Timer hat ausgelst"-Event bekommt.
   * 
   */
  private ActionListener timerTickListener;

  /**Dem Konstruktor wird das Interval des Timers bergeben.
   * Der Timer wird noch nicht gestartet - das erfolgt erst durch Aufrufen
   * von "start" 
   * 
   * @param intInterval Intervall des timers in Millisekunden - siehe Konstruktorparameter
   * des @see {@link javax.swing.Timer}
   */
  public SwingTimer (int intInterval)
  {
    this.timer = new Timer(intInterval, new ActionListener ()
    {
      @Override
      public void actionPerformed(ActionEvent e)
      {

        tick();
      }
    });
   
  }
  
  /**Wird ausgelst beim Timertick: ruft die actionPerformed-Methode
   * des per "addTimerListener" bergebenen ActionListener auf.
   * Wenn keiner da ist, passiert nichts.
   * 
   */
  private void tick()
  {
    if (this.timerTickListener != null)
    {
      this.timerTickListener.actionPerformed(null);
    }
  }

  /**{@inheritDoc}
   * 
   */
  @Override
  public void addTimerListener(ActionListener actionListener)
  {
    this.timerTickListener = actionListener;
    
  }
  
  /**Starten des Timers: hier luft der Swing-Timer los
   * und ruft alle x Millisekunden die "actionPerformed"-Methode
   * des ber "addTimerListener" gesetzten ActionListener auf. 
   * 
   * @exception IllegalStateException Wenn noch kein Listener gesetzt wurde, wird eine Exception geworfen.
   * 
   */
  @Override
  public void start()
  {
    if (this.timerTickListener == null)
    {
      throw new IllegalStateException("Kein Tick-Listener registriert - Timer kann nicht gestartet werden");
    }
    this.timer.start();
  }
  
}
